@@ -4,6 +4,8 @@
 # Description:  Apple EXIF maker notes tags
 #
 # Revisions:    2013-09-13 - P. Harvey Created
+#
+# References:   1) http://www.photoinvestigator.co/blog/the-mystery-of-maker-apple-metadata/
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Apple;
@@ -13,30 +15,48 @@ use vars qw($VERSION);
 use Image::ExifTool::Exif;
 use Image::ExifTool::PLIST;
 
-$VERSION = '1.02';
+$VERSION = '1.04';
 
+# Apple iPhone metadata (ref PH)
 %Image::ExifTool::Apple::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
     WRITABLE => 1,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
-    NOTES => 'Tags extracted from maker notes of images from the iPhone 5 with iOS 7.',
-    # 0x0001 - int32s: seen 0,1,2,3,4
+    NOTES => 'Tags extracted from the maker notes of iPhone images.',
+    # 0x0001 - int32s: seen 0,1,2,3,4,9
     # 0x0002 - binary plist with a single data object of size 512 bytes (iPhone5s)
     0x0003 => {
-        Name => 'RunTime',
+        Name => 'RunTime', # (includes time plugged in, but not when suspended, ref 1)
         SubDirectory => { TagTable => 'Image::ExifTool::Apple::RunTime' },
     },
     # 0x0004 - int32s: normally 1, but 0 for low-light images
-    # 0x0005 - int32s: seen values 147-247, and 100 for blank images
-    # 0x0006 - int32s: seen values 120-258, and 20 for blank images
+    # 0x0005 - int32s: seen values 113-247, and 100 for blank images
+    # 0x0006 - int32s: seen values 27-258, and 20 for blank images
     # 0x0007 - int32s: seen 1
-    # 0x0008 - rational64s[3]: eg) "0.02683717579 -0.7210501641 -0.6948792783"
-    # 0x0009 - int32s: seen 19,531
+    0x0008 => { #1
+        Name => 'AccelerationVector',
+        Groups => { 2 => 'Camera' },
+        Writable => 'rational64s',
+        Count => 3,
+        # Note: the directions are contrary to the Apple documentation (which have the
+        # signs of all axes reversed -- apparently the Apple geeks aren't very good
+        # with basic physics, and don't understand the concept of acceleration.  See
+        # http://nscookbook.com/2013/03/ios-programming-recipe-19-using-core-motion-to-access-gyro-and-accelerometer/
+        # for one of the few correct descriptions of this).  Note that this leads to
+        # a left-handed coordinate system for acceleration.
+        Notes => q{
+            XYZ coordinates of the acceleration vector in units of g.  As viewed from
+            the front of the phone, positive X is toward the left side, positive Y is
+            toward the bottom, and positive Z points into the face of the phone
+        },
+    },
+    # 0x0009 - int32s: seen 19,275,531,4371
     0x000a => {
         Name => 'HDRImageType',
         Writable => 'int32s',
         PrintConv => {
+            # 2 => ? (iPad mini 2)
             3 => 'HDR Image',
             4 => 'Original Image',
         },
@@ -47,12 +67,24 @@ $VERSION = '1.02';
         Notes => 'unique ID for all images in a burst',
     },
     # 0x000c - rational64s[2]: eg) "0.1640625 0.19921875"
-    # 0x000d - int32s: 0,1,6
-    # 0x000e - int32s: 0,1,12
+    # 0x000d - int32s: 0,1,6,20,24,32,40
+    # 0x000e - int32s: 0,1,4,12 (Orientation? 0=landscape? 4=portrait? ref 1)
     # 0x000f - int32s: 2,3
     # 0x0010 - int32s: 1
-    # 0x0011 - string[37]: some type of UID, eg. "FFCBAC24-E547-4BBC-AF47-38B1A3D845E3\0" (iPhone 6s, iOS 6.1)
-    # 0x0014 - int32s: 1,2,3,5 (iPhone 6s, iOS 6.1)
+    0x0011 => {
+        Name => 'ContentIdentifier', #forum8750
+        Writable => 'string',
+    },
+    # 0x0014 - int32s: 1,2,3,4,5 (iPhone 6s, iOS 6.1)
+    0x0015 => {
+        Name => 'ImageUniqueID',
+        Writable => 'string',
+    },
+    # 0x0016 - string[29]: "AXZ6pMTOh2L+acSh4Kg630XCScoO\0"
+    # 0x0017 - int32s: 0,8192
+    # 0x0019 - int32s: 0,2,128
+    # 0x001a - string[6]: "q825s\0"
+    # 0x001f - int32s: 0
 );
 
 # PLIST-format CMTime structure (ref PH)
@@ -116,7 +148,7 @@ Apple maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2016, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
