@@ -21,7 +21,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::ASF;   # for GetGUID()
 
-$VERSION = '1.36';
+$VERSION = '1.38';
 
 sub ProcessFPX($$);
 sub ProcessFPXR($$$);
@@ -125,7 +125,7 @@ my @dirEntryType = qw(INVALID STORAGE STREAM LOCKBYTES PROPERTY ROOT);
 # list of code pages used by Microsoft
 # (ref http://msdn.microsoft.com/en-us/library/dd317756(VS.85).aspx)
 my %codePage = (
-    037 => 'IBM EBCDIC US-Canada',
+     37 => 'IBM EBCDIC US-Canada',
     437 => 'DOS United States',
     500 => 'IBM EBCDIC International',
     708 => 'Arabic (ASMO 708)',
@@ -313,7 +313,8 @@ my %fpxFileType = (
         from DOC, PPT, XLS (Microsoft Word, PowerPoint and Excel) documents, VSD
         (Microsoft Visio) drawings, and FLA (Macromedia/Adobe Flash project) files
         since these are based on the same file format as FlashPix (the Windows
-        Compound Binary File format).  See
+        Compound Binary File format).  Note that ExifTool identifies any
+        unrecognized Windows Compound Binary file as a FlashPix (FPX) file.  See
         L<http://graphcomp.com/info/specs/livepicture/fpx.pdf> for the FlashPix
         specification.
     },
@@ -1085,7 +1086,7 @@ my %fpxFileType = (
             0x0807 => 'German (Swiss)',
             0x0408 => 'Greek',
             0x0409 => 'English (US)',
-            0x0809 => 'English (British)',		
+            0x0809 => 'English (British)',
             0x0c09 => 'English (Australian)',
             0x040a => 'Spanish (Castilian)',
             0x080a => 'Spanish (Mexican)',
@@ -1108,7 +1109,7 @@ my %fpxFileType = (
             0x0415 => 'Polish',
             0x0416 => 'Portuguese (Brazilian)',
             0x0816 => 'Portuguese',
-            0x0417 => 'Rhaeto-Romanic',	
+            0x0417 => 'Rhaeto-Romanic',
             0x0418 => 'Romanian',
             0x0419 => 'Russian',
             0x041a => 'Croato-Serbian (Latin)',
@@ -1175,11 +1176,11 @@ my %fpxFileType = (
     VARS => { NO_ID => 1 },
     CommentBy => {
         Groups => { 2 => 'Author' },
-        Notes => 'enable Duplicates option to extract all entries',
+        Notes => 'enable L<Duplicates|../ExifTool.html#Duplicates> option to extract all entries',
     },
     LastSavedBy => {
         Groups => { 2 => 'Author' },
-        Notes => 'enable Duplicates option to extract history of up to 10 entries',
+        Notes => 'enable L<Duplicates|../ExifTool.html#Duplicates> option to extract history of up to 10 entries',
     },
     DOP => { SubDirectory => { TagTable => 'Image::ExifTool::FlashPix::DOP' } },
     ModifyDate => {
@@ -1439,7 +1440,7 @@ sub ReadFPXValue($$$$$;$$)
                     my $charset = $Image::ExifTool::charsetName{"cp$codePage"};
                     if ($charset) {
                         $val = $et->Decode($val, $charset);
-                    } elsif ($codePage eq 1200) {   # UTF-16, little endian
+                    } elsif ($codePage == 1200) {   # UTF-16, little endian
                         $val = $et->Decode($val, 'UCS2', 'II');
                     }
                 }
@@ -1577,14 +1578,16 @@ sub ProcessDocumentTable($)
         my $key = 'TableOffsets' . ($i ? " ($i)" : '');
         my $offsets = $$value{$key};
         last unless defined $offsets;
-        my $doc = $$extra{$key}{G3} if $$extra{$key};
+        my $doc;
+        $doc = $$extra{$key}{G3} if $$extra{$key};
         $doc = '' unless $doc;
         # get DocFlags for this sub-document
         my ($docFlags, $docTable);
         for ($j=0; ; ++$j) {
             my $key = 'DocFlags' . ($j ? " ($j)" : '');
             last unless defined $$value{$key};
-            my $tmp = $$extra{$key}{G3} if $$extra{$key};
+            my $tmp;
+            $tmp = $$extra{$key}{G3} if $$extra{$key};
             $tmp = '' unless $tmp;
             if ($tmp eq $doc) {
                 $docFlags = $$value{$key};
@@ -1597,7 +1600,8 @@ sub ProcessDocumentTable($)
         for ($j=0; ; ++$j) {
             my $key = $tag . ($j ? " ($j)" : '');
             last unless defined $$value{$key};
-            my $tmp = $$extra{$key}{G3} if $$extra{$key};
+            my $tmp;
+            $tmp = $$extra{$key}{G3} if $$extra{$key};
             $tmp = '' unless $tmp;
             if ($tmp eq $doc) {
                 $docTable = \$$value{$key};
@@ -1642,14 +1646,14 @@ sub ProcessCommentBy($$$)
     my $pos = $$dirInfo{DirStart};
     my $end = $$dirInfo{DirLen} + $pos;
     $et->VerboseDir($$dirInfo{DirName});
-	while ($pos + 2 < $end) {
-		my $len = Get16u($dataPt, $pos);
-		$pos += 2;
-	    last if $pos + $len * 2 > $end;
-		my $author = $et->Decode(substr($$dataPt, $pos, $len*2), 'UCS2');
-		$pos += $len * 2;
-		$et->HandleTag($tagTablePtr, CommentBy => $author);
-	}
+    while ($pos + 2 < $end) {
+        my $len = Get16u($dataPt, $pos);
+        $pos += 2;
+        last if $pos + $len * 2 > $end;
+        my $author = $et->Decode(substr($$dataPt, $pos, $len*2), 'UCS2');
+        $pos += $len * 2;
+        $et->HandleTag($tagTablePtr, CommentBy => $author);
+    }
     return 1;
 }
 
@@ -1665,24 +1669,24 @@ sub ProcessLastSavedBy($$$)
     my $end = $$dirInfo{DirLen} + $pos;
     return 0 if $pos + 6 > $end;
     $et->VerboseDir($$dirInfo{DirName});
-	my $num = Get16u($dataPt, $pos+2);
-	$pos += 6;
-	while ($num >= 2) {
-	    last if $pos + 2 > $end;
-		my $len = Get16u($dataPt, $pos);
-		$pos += 2;
-	    last if $pos + $len * 2 > $end;
-		my $author = $et->Decode(substr($$dataPt, $pos, $len*2), 'UCS2');
-		$pos += $len * 2;
-	    last if $pos + 2 > $end;
-		$len = Get16u($dataPt, $pos);
-		$pos += 2;
-	    last if $pos + $len * 2 > $end;
-		my $path = $et->Decode(substr($$dataPt, $pos, $len*2), 'UCS2');
-		$pos += $len * 2;
-		$et->HandleTag($tagTablePtr, LastSavedBy => "$author ($path)");
-		$num -= 2;
-	}
+    my $num = Get16u($dataPt, $pos+2);
+    $pos += 6;
+    while ($num >= 2) {
+        last if $pos + 2 > $end;
+        my $len = Get16u($dataPt, $pos);
+        $pos += 2;
+        last if $pos + $len * 2 > $end;
+        my $author = $et->Decode(substr($$dataPt, $pos, $len*2), 'UCS2');
+        $pos += $len * 2;
+        last if $pos + 2 > $end;
+        $len = Get16u($dataPt, $pos);
+        $pos += 2;
+        last if $pos + $len * 2 > $end;
+        my $path = $et->Decode(substr($$dataPt, $pos, $len*2), 'UCS2');
+        $pos += $len * 2;
+        $et->HandleTag($tagTablePtr, LastSavedBy => "$author ($path)");
+        $num -= 2;
+    }
     return 1;
 }
 
@@ -1995,7 +1999,7 @@ sub ProcessFPXR($$$)
             $et->Warn("Unlisted FPXR segment (index $index)") if $index != 255;
         }
 
-    } elsif ($type ne 3) {  # not a "Reserved" segment
+    } elsif ($type != 3) {  # not a "Reserved" segment
 
         $et->Warn("Unknown FPXR segment (type $type)");
 
@@ -2220,7 +2224,7 @@ sub ProcessFPX($$)
         my $rSib = Get32u(\$dir, $pos + 0x48);  # right sibling
         my $chld = Get32u(\$dir, $pos + 0x4c);  # child directory
 
-        # save information about object hierachy
+        # save information about object hierarchy
         my ($obj, $sub);
         $obj = $hier{$index} or $obj = $hier{$index} = { };
         $$obj{Left} = $lSib unless $lSib == FREE_SECT;
@@ -2366,7 +2370,7 @@ JPEG images.
 
 =head1 AUTHOR
 
-Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
