@@ -14,7 +14,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.03';
+$VERSION = '1.04';
 
 sub ReadBencode($$);
 sub ExtractTags($$$;$$@);
@@ -41,7 +41,10 @@ sub ExtractTags($$$;$$@);
         PrintConv => '$self->ConvertDateTime($val)',
     },
     'encoding'      => { },
-    'info'          => { SubDirectory => { TagTable => 'Image::ExifTool::Torrent::Info' } },
+    'info'          => {
+        SubDirectory => { TagTable => 'Image::ExifTool::Torrent::Info' },
+        Notes => 'extracted as a structure with the Struct option',
+    },
     'url-list'      => { Name => 'URLList1' },
 );
 
@@ -198,7 +201,7 @@ sub ExtractTags($$$;$$@)
                 $name = "Tag$name" if length($name) < 2 or $name !~ /^[A-Z]/;
                 $name = $baseName . $name if defined $baseName; # add base name if necessary
                 AddTagToTable($tagTablePtr, $id, { Name => $name });
-                $et->VPrint(0, "  [adding $id '$name']\n");
+                $et->VPrint(0, "  [adding $id '${name}']\n");
             }
             my $tagInfo = $et->GetTagInfo($tagTablePtr, $id) or next;
             if (ref $val eq 'ARRAY') {
@@ -232,6 +235,11 @@ sub ExtractTags($$$;$$@)
                 $tagInfo = $et->GetTagInfo($tagTablePtr, $id) or next;
             }
             if (ref $val eq 'HASH') {
+                if ($et->Options('Struct') and $tagInfo and $$tagInfo{Name} eq 'Info') {
+                    $et->FoundTag($tagInfo, $val);
+                    ++$count;
+                    next;
+                }
                 # extract tags from this dictionary
                 my ($table, $rootID, $rootName);
                 if ($$tagInfo{SubDirectory}) {
@@ -268,7 +276,7 @@ sub ProcessTorrent($$)
     my $dict = ReadBencode($raf, \$buff);
     my $err = $$raf{BencodeError};
     $et->Warn("Bencode error: $err") if $err;
-    if (ref $dict eq 'HASH' and $$dict{announce}) {
+    if (ref $dict eq 'HASH' and ($$dict{announce} or $$dict{'created by'})) {
         $et->SetFileType();
         my $tagTablePtr = GetTagTable('Image::ExifTool::Torrent::Main');
         ExtractTags($et, $dict, $tagTablePtr) and $success = 1;
@@ -295,7 +303,7 @@ bencoded information from BitTorrent files.
 
 =head1 AUTHOR
 
-Copyright 2003-2016, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
