@@ -284,8 +284,9 @@ sub WritePDF($$)
 
     # make sure this is a PDF file
     my $pos = $raf->Tell();
-    $raf->Read($buff, 10) >= 8 or return 0;
-    $buff =~ /^%PDF-(\d+\.\d+)/ or return 0;
+    $raf->Read($buff, 1024) >= 8 or return 0;
+    $buff =~ /^(\s*)%PDF-(\d+\.\d+)/ or return 0;
+    $$et{PDFBase} = length $1;
     $raf->Seek($pos, 0);
 
     # create a new ExifTool object and use it to read PDF and XMP information
@@ -297,15 +298,13 @@ sub WritePDF($$)
     # not a valid PDF file unless we got a version number
     # (note: can't just check $$info{PDFVersion} due to possibility of XMP-pdf:PDFVersion)
     my $vers = $newTool->GetInfo('PDF:PDFVersion');
-    ($pdfVer) = values %$vers;
+    # take highest version number if multiple versions in an incremental save
+    ($pdfVer) = sort { $b <=> $a } values %$vers;
     $pdfVer or $et->Error('Missing PDF:PDFVersion'), return 0;
     # check version number
     if ($pdfVer > 1.7) {
-        if ($pdfVer >= 2.0) {
-            $et->Error("Can't yet write PDF version $pdfVer"); # (future major version changes)
-            return 1;
-        }
-        $et->Warn("ExifTool is untested with PDF version $pdfVer files", 1);
+        $et->Warn("The PDF $pdfVer specification is not freely available", 1);
+        # (so writing by ExifTool is based on trial and error)
     }
     # fail if we had any serious errors while extracting information
     if ($capture{Error} or $$info{Error}) {
@@ -331,13 +330,13 @@ sub WritePDF($$)
     if ($buff =~ /$endComment(\d+)\s+(startxref\s+\d+\s+%%EOF\s+)?$/s) {
         $prevUpdate = $1;
         # rewrite the file up to the original EOF
-        Image::ExifTool::CopyBlock($raf, $outfile, $prevUpdate) or $rtn = -1;
+        Image::ExifTool::CopyBlock($raf, $outfile, $prevUpdate + $$et{PDFBase}) or $rtn = -1;
         # verify that we are now at the start of an ExifTool update
         unless ($raf->Read($buff, length $beginComment) and $buff eq $beginComment) {
             $et->Error('Previous ExifTool update is corrupted');
             return $rtn;
         }
-        $raf->Seek($prevUpdate, 0) or $rtn = -1;
+        $raf->Seek($prevUpdate+$$et{PDFBase}, 0) or $rtn = -1;
         if ($$et{DEL_GROUP}{'PDF-update'}) {
             $et->VPrint(0, "  Reverted previous ExifTool updates\n");
             ++$$et{CHANGED};
@@ -570,7 +569,7 @@ sub WritePDF($$)
 #
     if ($$et{CHANGED}) {
         # remember position of original EOF
-        my $oldEOF = Tell($outfile);
+        my $oldEOF = Tell($outfile) - $$et{PDFBase};
         Write($outfile, $beginComment) or $rtn = -1;
 
         # write new objects
@@ -584,7 +583,7 @@ sub WritePDF($$)
                 next;
             }
             # create new entry for xref table
-            $newXRef{$id} = [ Tell($outfile) + length($/), $gen, 'n' ];
+            $newXRef{$id} = [ Tell($outfile) - $$et{PDFBase} + length($/), $gen, 'n' ];
             $keyExt = "$id $gen obj";  # (must set for stream encryption)
             Write($outfile, $/, $keyExt) or $rtn = -1;
             WriteObject($outfile, $newObj{$objRef}) or $rtn = -1;
@@ -645,13 +644,13 @@ sub WritePDF($$)
         }
 
         # remember position of xref table in file (we will write this next)
-        my $startxref = Tell($outfile) + length($/);
+        my $startxref = Tell($outfile) - $$et{PDFBase} + length($/);
 
         # must write xref as a stream in xref-stream-only files
         if ($$mainDict{Type} and $$mainDict{Type} eq '/XRef') {
 
             # create entry for the xref stream object itself
-            $newXRef{$nextObject++} = [ Tell($outfile) + length($/), 0, 'n' ];
+            $newXRef{$nextObject++} = [ Tell($outfile) - $$et{PDFBase} + length($/), 0, 'n' ];
             $$mainDict{Size} = $nextObject;
             # create xref stream and Index entry
             $$mainDict{W} = [ 1, 4, 2 ];    # int8u, int32u, int16u ('CNn')
@@ -711,7 +710,7 @@ sub WritePDF($$)
     } elsif ($prevUpdate) {
 
         # nothing new changed, so copy over previous incremental update
-        $raf->Seek($prevUpdate, 0) or $rtn = -1;
+        $raf->Seek($prevUpdate+$$et{PDFBase}, 0) or $rtn = -1;
         while ($raf->Read($buff, 65536)) {
             Write($outfile, $buff) or $rtn = -1;
         }
@@ -750,7 +749,7 @@ C<PDF-update> pseudo group).
 
 =head1 AUTHOR
 
-Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
