@@ -37,7 +37,7 @@ my %mp4Map = (
     Track     => 'Movie',
 );
 my %heicMap = (
-    # HEIC ('ftyp' compatible brand 'heic' or 'mif1') -> XMP/EXIF in top level 'meta'
+    # HEIC/HEIF/AVIF ('ftyp' compatible brand 'heic','mif1','avif') -> XMP/EXIF in top level 'meta'
     Meta         => 'MOV',
     ItemInformation => 'Meta',
     ItemPropertyContainer => 'Meta',
@@ -82,6 +82,8 @@ my %qtFormat = (
 );
 my $undLang = 0x55c4;   # numeric code for default ('und') language
 
+my $maxReadLen = 100000000; # maximum size of atom to read into memory (100 MB)
+
 # boxes that may exist in an "empty" Meta box:
 my %emptyMeta = (
     hdlr => 'Handler', 'keys' => 'Keys', lang => 'Language', ctry => 'Country', free => 'Free',
@@ -156,9 +158,12 @@ sub ConvInvISO6709($)
     my $val = shift;
     my @a = split ' ', $val;
     if (@a == 2 or @a == 3) {
+        # latitude must have 2 digits before the decimal, and longitude 3,
+        # and all values must start with a "+" or "-"
+        my @fmt = ('%s%02d','%s%03d','%s%d');
         foreach (@a) {
-            Image::ExifTool::IsFloat($_) or return undef;
-            $_ = '+' . $_ if $_ >= 0;
+            return undef unless Image::ExifTool::IsFloat($_);
+            $_ =~ s/^([-+]?)(\d+)/sprintf(shift(@fmt), $1 || '+', $2)/e;
         }
         return join '', @a;
     }
@@ -444,7 +449,7 @@ sub WriteKeys($$$)
 sub WriteItemInfo($$$)
 {
     my ($et, $dirInfo, $outfile) = @_;
-    my $boxPos = $$dirInfo{BoxPos};
+    my $boxPos = $$dirInfo{BoxPos};     # hash of [length,position] for each box
     my $raf = $$et{RAF};
     my $items = $$et{ItemInfo};
     my (%did, @mdatEdit, $name);
@@ -491,7 +496,11 @@ sub WriteItemInfo($$$)
             $buff = $val . $buff if length $val;
             my ($hdr, $subTable, $proc);
             if ($name eq 'EXIF') {
-                $hdr = "\0\0\0\x06Exif\0\0";
+                if (length($buff) < 4 or length($buff) < 4 + unpack('N',$buff)) {
+                    $et->Warn('Invalid Exif header');
+                    next;
+                }
+                $hdr = substr($buff, 0, 4 + unpack('N',$buff));
                 $subTable = GetTagTable('Image::ExifTool::Exif::Main');
                 $proc = \&Image::ExifTool::WriteTIFF;
             } else {
@@ -523,7 +532,7 @@ sub WriteItemInfo($$$)
                     } elsif ($n <= 0xffffffff) {
                         Set32u($n, $outfile, $$boxPos{iloc}[0] + 8 + $lenPt);
                     } else {
-                        $et->Error("Can't yet promote iloc offset to 64 bits");
+                        $et->Error("Can't yet promote iloc length to 64 bits");
                         return ();
                     }
                     $n = 0;
@@ -540,8 +549,11 @@ sub WriteItemInfo($$$)
     my ($countNew, %add, %usedID);
     foreach $name ('EXIF','XMP') {
         next if $did{$name} or not $$et{ADD_DIRS}{$name};
-        unless ($$boxPos{iinf} and $$boxPos{iref} and $$boxPos{iloc}) {
-            $et->Warn("Can't create $name. Missing expected box");
+        my @missing;
+        $$boxPos{$_} or push @missing, $_ foreach qw(iinf iloc);
+        if (@missing) {
+            my $str = @missing > 1 ? join(' and ', @missing) . ' boxes' : "@missing box";
+            $et->Warn("Can't create $name. Missing expected $str");
             last;
         }
         my $primary = $$et{PrimaryItem};
@@ -568,6 +580,14 @@ sub WriteItemInfo($$$)
         my $changed = $$et{CHANGED};
         my $newVal = $et->WriteDirectory(\%dirInfo, $subTable, $proc);
         if (defined $newVal and $changed ne $$et{CHANGED}) {
+            my $irefVer;
+            if ($$boxPos{iref}) {
+                $irefVer = Get8u($outfile, $$boxPos{iref}[0] + 8);
+            } else {
+                # create iref box after end of iinf box (and save version in boxPos list)
+                $irefVer = ($primary > 0xffff ? 1 : 0);
+                $$boxPos{iref} = [ $$boxPos{iinf}[0] + $$boxPos{iinf}[1], 0, $irefVer ];
+            }
             $newVal = $hdr . $newVal if length $hdr;
             # add new infe to iinf
             $add{iinf} = $add{iref} = $add{iloc} = '' unless defined $add{iinf};
@@ -589,7 +609,6 @@ sub WriteItemInfo($$$)
                 $add{iinf} .= pack('Na4CCCCNn', $n, 'infe', 3, 0, 0, 1, $id, 0) . $type . $mime;
             }
             # add new cdsc to iref
-            my $irefVer = Get8u($outfile, $$boxPos{iref}[0] + 8);
             if ($irefVer) {
                 $add{iref} .= pack('Na4NnN', 18, 'cdsc', $id, 1, $primary);
             } else {
@@ -602,18 +621,21 @@ sub WriteItemInfo($$$)
             my $nlen = ($siz >> 8) & 0x0f;
             my $nbas = ($siz >> 4) & 0x0f;
             my $nind = $siz & 0x0f;
-            my $p;
+            my ($pbas, $poff);
             if ($ilocVer == 0) {
                 # set offset to 0 as flag that this is a new idat chunk being added
-                $p = length($add{iloc}) + 4 + $nbas + 2;
+                $pbas = length($add{iloc}) + 4;
+                $poff = $pbas + $nbas + 2;
                 $add{iloc} .= pack('nn',$id,0) . SetVarInt(0,$nbas) . Set16u(1) .
                             SetVarInt(0,$noff) . SetVarInt(length($newVal),$nlen);
             } elsif ($ilocVer == 1) {
-                $p = length($add{iloc}) + 6 + $nbas + 2 + $nind;
+                $pbas = length($add{iloc}) + 6;
+                $poff = $pbas + $nbas + 2 + $nind;
                 $add{iloc} .= pack('nnn',$id,0,0) . SetVarInt(0,$nbas) . Set16u(1) . SetVarInt(0,$nind) .
                             SetVarInt(0,$noff) . SetVarInt(length($newVal),$nlen);
             } elsif ($ilocVer == 2) {
-                $p = length($add{iloc}) + 8 + $nbas + 2 + $nind;
+                $pbas = length($add{iloc}) + 8;
+                $poff = $pbas + $nbas + 2 + $nind;
                 $add{iloc} .= pack('Nnn',$id,0,0) . SetVarInt(0,$nbas) . Set16u(1) . SetVarInt(0,$nind) .
                             SetVarInt(0,$noff) . SetVarInt(length($newVal),$nlen);
             } else {
@@ -624,9 +646,19 @@ sub WriteItemInfo($$$)
             my $off = $$dirInfo{ChunkOffset} or $et->Warn('Internal error. Missing ChunkOffset'), last;
             my $newOff;
             if ($noff == 4) {
-                $newOff = [ 'stco_iloc', $$boxPos{iloc}[0] + $$boxPos{iloc}[1] + $p, $noff, 0, $id ];
+                $newOff = [ 'stco_iloc', $$boxPos{iloc}[0] + $$boxPos{iloc}[1] + $poff, $noff, 0, $id ];
             } elsif ($noff == 8) {
-                $newOff = [ 'co64_iloc', $$boxPos{iloc}[0] + $$boxPos{iloc}[1] + $p, $noff, 0, $id ];
+                $newOff = [ 'co64_iloc', $$boxPos{iloc}[0] + $$boxPos{iloc}[1] + $poff, $noff, 0, $id ];
+            } elsif ($noff == 0) {
+                # offset_size is zero, so store the offset in base_offset instead
+                if ($nbas == 4) {
+                    $newOff = [ 'stco_iloc', $$boxPos{iloc}[0] + $$boxPos{iloc}[1] + $pbas, $nbas, 0, $id ];
+                } elsif ($nbas == 8) {
+                    $newOff = [ 'co64_iloc', $$boxPos{iloc}[0] + $$boxPos{iloc}[1] + $pbas, $nbas, 0, $id ];
+                } else {
+                    $et->Warn("Can't create $name. Invalid iloc offset+base size");
+                    last;
+                }
             } else {
                 $et->Warn("Can't create $name. Invalid iloc offset size");
                 last;
@@ -646,15 +678,22 @@ sub WriteItemInfo($$$)
         foreach $tag (sort { $$boxPos{$a}[0] <=> $$boxPos{$b}[0] } keys %$boxPos) {
             next unless $add{$tag};
             my $pos = $$boxPos{$tag}[0] + $added;
-            my $n = Get32u($outfile, $pos);
-            Set32u($n + length($add{$tag}), $outfile, $pos);        # increase box size
+            unless ($$boxPos{$tag}[1]) {
+                $tag eq 'iref' or $et->Error('Internal error adding iref box'), last;
+                # create new iref box
+                $add{$tag} = Set32u(12 + length $add{$tag}) . $tag .
+                             Set8u($$boxPos{$tag}[2]) . "\0\0\0" . $add{$tag};
+            } else {
+                my $n = Get32u($outfile, $pos);
+                Set32u($n + length($add{$tag}), $outfile, $pos);    # increase box size
+            }
             if ($tag eq 'iinf') {
                 my $iinfVer = Get8u($outfile, $pos + 8);
                 if ($iinfVer == 0) {
-                    $n = Get16u($outfile, $pos + 12);
+                    my $n = Get16u($outfile, $pos + 12);
                     Set16u($n + $countNew, $outfile, $pos + 12);    # incr count
                 } else {
-                    $n = Get32u($outfile, $pos + 12);
+                    my $n = Get32u($outfile, $pos + 12);
                     Set32u($n + $countNew, $outfile, $pos + 12);    # incr count
                 }
             } elsif ($tag eq 'iref') {
@@ -662,10 +701,10 @@ sub WriteItemInfo($$$)
             } elsif ($tag eq 'iloc') {
                 my $ilocVer = Get8u($outfile, $pos + 8);
                 if ($ilocVer < 2) {
-                    $n = Get16u($outfile, $pos + 14);
+                    my $n = Get16u($outfile, $pos + 14);
                     Set16u($n + $countNew, $outfile, $pos + 14);    # incr count
                 } else {
-                    $n = Get32u($outfile, $pos + 14);
+                    my $n = Get32u($outfile, $pos + 14);
                     Set32u($n + $countNew, $outfile, $pos + 14);    # incr count
                 }
                 # must also update pointer locations in this box
@@ -697,7 +736,7 @@ sub WriteQuickTime($$$)
     my ($et, $dirInfo, $tagTablePtr) = @_;
     $et or return 1;    # allow dummy access to autoload this package
     my ($mdat, @mdat, @mdatEdit, $edit, $track, $outBuff, $co, $term, $delCount);
-    my (%langTags, $canCreate, $delGrp, %boxPos, %didDir, $writeLast, $err);
+    my (%langTags, $canCreate, $delGrp, %boxPos, %didDir, $writeLast, $err, $atomCount);
     my $outfile = $$dirInfo{OutFile} || return 0;
     my $raf = $$dirInfo{RAF};       # (will be null for lower-level atoms)
     my $dataPt = $$dirInfo{DataPt}; # (will be null for top-level atoms)
@@ -768,7 +807,14 @@ sub WriteQuickTime($$$)
         $canCreate = 1;
         $delGrp = $$et{DEL_GROUP}{$dirName};
     }
+    $atomCount = $$tagTablePtr{VARS}{ATOM_COUNT} if $$tagTablePtr{VARS};
+
     for (;;) {      # loop through all atoms at this level
+        if (defined $atomCount and --$atomCount < 0 and $dataPt) {
+            # stop processing now and just copy the rest of the atom
+            Write($outfile, substr($$dataPt, $raf->Tell())) or $rtnVal=$rtnErr, $err=1;
+            last;
+        }
         my ($hdr, $buff, $keysIndex);
         my $n = $raf->Read($hdr, 8);
         unless ($n == 8) {
@@ -801,6 +847,7 @@ sub WriteQuickTime($$$)
             $size < 0 and $et->Error('Invalid extended atom size'), last;
         } elsif ($size == -8) {
             if ($dataPt) {
+                last if $$dirInfo{DirName} eq 'CanonCNTH';  # (this is normal for Canon CNTH atom)
                 my $pos = $raf->Tell() - 4;
                 $raf->Seek(0,2);
                 my $str = $$dirInfo{DirName} . ' with ' . ($raf->Tell() - $pos) . ' bytes';
@@ -813,15 +860,8 @@ sub WriteQuickTime($$$)
             }
             last;
         } elsif ($size < 0) {
-            if ($$tagTablePtr{VARS}{IGNORE_BAD_ATOMS} and $dataPt) {
-                # ignore bad atom and just copy the rest of this directory
-                $buff = substr($$dataPt, $raf->Tell());
-                Write($outfile, $hdr, $buff) or $rtnVal=$rtnErr, $err=1;
-                last;
-            } else {
-                $et->Error('Invalid atom size');
-                last;
-            }
+            $et->Error('Invalid atom size');
+            last;
         }
 
         # keep track of 'mdat' atom locations for writing later
@@ -841,17 +881,35 @@ sub WriteQuickTime($$$)
         }
 
         # read the atom data
+        my $got;
         if (not $size) {
             $buff = '';
-        } elsif ($size > 100000000) {
-            my $mb = int($size / 0x100000 + 0.5);
-            $tag = PrintableTagID($tag,3);
-            $et->Error("'${tag}' atom is too large for rewriting ($mb MB)");
-            return $rtnVal;
-        } elsif ($raf->Read($buff, $size) != $size) {
-            $tag = PrintableTagID($tag,3);
-            $et->Error("Truncated $tag atom");
-            return $rtnVal;
+            $got = 0;
+        } else {
+            # read the atom data (but only first 64kB if data is huge)
+            $got = $raf->Read($buff, $size > $maxReadLen ? 0x10000 : $size);
+        }
+        if ($got != $size) {
+            # ignore up to 256 bytes of garbage at end of file
+            if ($got <= 256 and $size >= 1024 and $tag ne 'mdat') {
+                my $bytes = $got + length $hdr;
+                if ($$et{OPTIONS}{IgnoreMinorErrors}) {
+                    $et->Warn("Deleted garbage at end of file ($bytes bytes)");
+                    $buff = $hdr = '';
+                } else {
+                    $et->Error("Possible garbage at end of file ($bytes bytes)", 1);
+                    return $rtnVal;
+                }
+            } else {
+                $tag = PrintableTagID($tag,3);
+                if ($size > $maxReadLen and $got == 0x10000) {
+                    my $mb = int($size / 0x100000 + 0.5);
+                    $et->Error("'${tag}' atom is too large for rewriting ($mb MB)");
+                } else {
+                    $et->Error("Truncated '${tag}' atom");
+                }
+                return $rtnVal;
+            }
         }
         # save the handler type for this track
         if ($tag eq 'hdlr' and length $buff >= 12) {
@@ -862,7 +920,7 @@ sub WriteQuickTime($$$)
         # if this atom stores offsets, save its location so we can fix up offsets later
         # (are there any other atoms that may store absolute file offsets?)
         if ($tag =~ /^(stco|co64|iloc|mfra|moof|sidx|saio|gps |CTBO|uuid)$/) {
-            # (note that we only need to do this if the movie data is stored in this file)
+            # (note that we only need to do this if the media data is stored in this file)
             my $flg = $$et{QtDataFlg};
             if ($tag eq 'mfra' or $tag eq 'moof') {
                 $et->Error("Can't yet handle movie fragments when writing");
@@ -890,7 +948,7 @@ sub WriteQuickTime($$$)
                 $et->Error("Can't locate data reference to update offsets for $grp");
                 return $rtnVal;
             } elsif ($flg == 3) {
-                $et->Error("Can't write files with mixed internal/external movie data");
+                $et->Error("Can't write files with mixed internal/external media data");
                 return $rtnVal;
             } elsif ($flg == 1) {
                 # must update offsets since the data is in this file
@@ -1048,7 +1106,7 @@ sub WriteQuickTime($$$)
                                         my $newVal = $et->GetNewValue($nvHash);
                                         next unless defined $newVal;
                                         my $prVal = $newVal;
-                                        my $flags = FormatQTValue($et, \$newVal, $$tagInfo{Format});
+                                        my $flags = FormatQTValue($et, \$newVal, $format);
                                         next unless defined $newVal;
                                         my ($ctry, $lang) = (0, $undLang);
                                         if ($$ti{LangCode}) {
@@ -1104,7 +1162,6 @@ sub WriteQuickTime($$$)
                                     $val =~ s/\0$// unless $$tagInfo{Binary};
                                     $flags = 0x01;  # write all strings as UTF-8
                                 } else {
-                                    $format = $$tagInfo{Format};
                                     if ($format) {
                                         # update flags for the format we are writing
                                         $flags = $qtFormat{$format} if $qtFormat{$format};
@@ -1126,7 +1183,7 @@ sub WriteQuickTime($$$)
                                     }
                                     my $prVal = $newVal;
                                     # format new value for writing (and get new flags)
-                                    $flags = FormatQTValue($et, \$newVal, $$tagInfo{Format});
+                                    $flags = FormatQTValue($et, \$newVal, $format);
                                     my $grp = $et->GetGroup($langInfo, 1);
                                     $et->VerboseValue("- $grp:$$langInfo{Name}", $val);
                                     $et->VerboseValue("+ $grp:$$langInfo{Name}", $prVal);
@@ -1200,11 +1257,14 @@ sub WriteQuickTime($$$)
                                 } else {
                                     $newData = pack('nn', length($newData), $lang) . $newData;
                                 }
-                            } elsif (not $$tagInfo{Format} or $$tagInfo{Format} =~ /^string/ and
-                                    not $$tagInfo{Binary} and not $$tagInfo{ValueConv})
+                            } elsif (not $format or $format =~ /^string/ and
+                                     not $$tagInfo{Binary} and not $$tagInfo{ValueConv})
                             {
                                 # write all strings as UTF-8
                                 $newData = $et->Encode($newData, 'UTF8');
+                            } elsif ($format and not $$tagInfo{Binary}) {
+                                # format new value for writing
+                                $newData = WriteValue($newData, $format);
                             }
                         }
                         $$didTag{$nvHash} = 1;   # set flag so we don't add this tag again
@@ -1242,11 +1302,11 @@ sub WriteQuickTime($$$)
         } elsif ($tag eq 'stsd' and length($buff) >= 8) {
             my $n = Get32u(\$buff, 4);      # get number of sample descriptions in table
             my ($pos, $flg) = (8, 0);
-            my $i;
+            my ($i, $msg);
             for ($i=0; $i<$n; ++$i) {       # loop through sample descriptions
-                last if $pos + 16 > length($buff);
+                $pos + 16 <= length($buff) or $msg = 'Truncated sample table', last;
                 my $siz = Get32u(\$buff, $pos);
-                last if $pos + $siz > length($buff);
+                $pos + $siz <= length($buff) or $msg = 'Truncated sample table', last;
                 my $drefIdx = Get16u(\$buff, $pos + 14);
                 my $drefTbl = $$et{QtDataRef};
                 if (not $drefIdx) {
@@ -1256,12 +1316,16 @@ sub WriteQuickTime($$$)
                     # $flg = 0x01-in this file, 0x02-in some other file
                     $flg |= ($$dref[1] == 1 and $$dref[0] ne 'rsrc') ? 0x01 : 0x02;
                 } else {
-                    my $grp = $$et{CUR_WRITE_GROUP} || $parent;
-                    $et->Error("No data reference for $grp sample description $i");
-                    return $rtnVal;
+                    $msg = "No data reference for sample description $i";
+                    last;
                 }
                 $pos += $siz;
             }
+            if ($msg) {
+                my $grp = $$et{CUR_WRITE_GROUP} || $parent;
+                $et->Error("$msg for $grp");
+                return $rtnErr;
+            }
             $$et{QtDataFlg} = $flg;
         }
         if ($tagInfo and $$tagInfo{WriteLast}) {
@@ -1477,10 +1541,10 @@ sub WriteQuickTime($$$)
     if (not @mdat) {
         foreach $co (@$off) {
             next if $$co[0] eq 'uuid';
-            $et->Error('Movie data referenced but not found');
+            $et->Error('Media data referenced but not found');
             return $rtnVal;
         }
-        $et->Warn('No movie data', 1);
+        $et->Warn('No media data', 1);
     }
 
     # edit mdat blocks as required
@@ -1679,7 +1743,7 @@ sub WriteQuickTime($$$)
                 last;
             }
             unless ($ok) {
-                $et->Error("Chunk offset in $tag atom is outside movie data");
+                $et->Error("Chunk offset in $tag atom is outside media data");
                 return $rtnVal;
             }
         }
@@ -1691,7 +1755,7 @@ sub WriteQuickTime($$$)
     # write the metadata
     Write($outfile, $outBuff) or $rtnVal = 0;
 
-    # write the movie data
+    # write the media data
     foreach $mdat (@mdat) {
         Write($outfile, $$mdat[2]) or $rtnVal = 0;  # write mdat header
         if ($$mdat[4]) {
@@ -1747,7 +1811,7 @@ sub WriteMOV($$)
     {
         if ($buff =~ /^crx /) {
             $ftype = 'CR3',
-        } elsif ($buff =~ /^(heic|mif1|msf1|heix|hevc|hevx)/) {
+        } elsif ($buff =~ /^(heic|mif1|msf1|heix|hevc|hevx|avif)/) {
             $ftype = 'HEIC';
         } else {
             $ftype = 'MP4';
@@ -1789,7 +1853,7 @@ QuickTime-based file formats like MOV and MP4.
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
